module bnhelix
! Define nonstationary filter type
  use helix
implicit none
  type bnfilter  
!  nd  is data length.
    logical,       dimension(:), pointer :: mis  
    ! (nd) boundary conditions
    integer,       dimension(:), pointer :: pch  ! (nd) patches
    integer,       dimension(:), pointer :: lag  ! (nh) lags (assumed same for all filters)
    real,          dimension(:,:), pointer :: flt  ! (nh,np) filter coefficients
  end type
  contains
  subroutine bnallocate( aa, nh, pch)
! allocate a filter
    type( bnfilter)                       :: aa
    integer, dimension(:), intent( in)   :: pch  
    integer                              :: ip, np, nd, nh
    np = maxval(pch)
    allocate( aa%lag(nh))
    allocate( aa%flt(nh,np))
    nd = size( pch)
    allocate( aa%pch(nd)) 
    aa%pch = pch
    nullify( aa%mis)				   ! set null pointer for mis.
  end subroutine 
  subroutine ndeallocate( aa)
! destroy a filter
    type( bnfilter) :: aa
    integer        :: ip
    deallocate( aa%lag, aa%pch, aa%flt)
    if ( associated( aa%mis)) then
! if logicals were allocated
      deallocate( aa%mis)			   ! free them
    end if
  end subroutine 
end module 
