module laplacf
! Laplacian operator on 3D filter
  use adj_mod
  use laplacian
  implicit none

  logical, parameter,     private               :: T = .true. 
  logical, parameter,     private               :: F = .false. 
  integer, dimension (:), allocatable, private  :: m
  integer,                             private  :: nlag,m123 
  real, dimension (:), allocatable              :: temp
  real, dimension (:,:),allocatable             :: x2,y2

  contains

  subroutine laplacf_init ( m_in,   nlag_in)
    integer            ::    m_in(:),nlag_in
!write(0,*)"ilaplacf_init 1"
    allocate (m(size(m_in)))
!write(0,*)"ilaplacf_init 2"
    m = m_in
!write(0,*)"ilaplacf_init 3",m
    nlag = nlag_in
!write(0,*)"ilaplacf_init 4",m,eps,na
    call laplac_init(m)
!write(0,*)"ilaplacf_init 5"
    m123=product(m)
!write(0,*)"ilaplacf_init 6",nlag,m123
    allocate(x2(nlag,m123),y2(nlag,m123),temp(nlag*m123))
!write(0,*)"ilaplacf_init 7"
  end subroutine laplacf_init

  function laplacf_lop ( adj,add,x,y) result(stat)
    integer            ::                      stat 
    logical,intent(in) ::  adj,add
    real,dimension(:)  ::          x,y 
    call adjnull (adj,add,x,y )
    call laplacf_lop2(adj,x,y )
    stat=0
  end function laplacf_lop

  subroutine laplacf_lop2 (adj,x,y)
    logical,intent(in)   :: adj
    real, dimension (:)  ::     x,y 
    integer :: stat1, ilag
!write(0,*)"laplacf_lop 1"
    if ( adj) then
!write(0,*)"laplacf_lop 2",size(y2),size(y),nlag,m123
       y2 = reshape(y,(/nlag,m123/))
!write(0,*)"laplacf_lop 3"
       do ilag=1,nlag
          stat1 = laplac_lop (  adj,  F, x2(ilag,:), y2(ilag,:))
       end do
!write(0,*)"laplacf_lop 4"
       temp = reshape(x2,(/nlag*m123/))
       x = x + temp
!write(0,*)"laplacf_lop 5"
    else
!write(0,*)"laplacf_lop 6",size(x),size(x2),nlag,m123
       x2 = reshape(x,(/nlag,m123/))
!write(0,*)"laplacf_lop 7"
       do ilag=1,nlag
          stat1 = laplac_lop (  adj,  F, x2(ilag,:), y2(ilag,:))
       end do
!write(0,*)"laplacf_lop 8",nlag,m123,size(y2),size(temp)
       temp = reshape(y2,(/nlag*m123/))
!write(0,*)"laplacf_lop 8.5",size(y),size(temp)
       y = y + temp
!write(0,*)"laplacf_lop 9"
    end if
  end subroutine laplacf_lop2
  subroutine laplacf_close()
    deallocate(x2,y2,temp,m)
    call laplac_close()
  end subroutine laplacf_close
end module laplacf

